﻿using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;

namespace WpfApplication1 {

    public static class CommandBehavior {

        // UI,Handler List
        private static Dictionary<UIElement, RoutedEventHandler> handlers = new Dictionary<UIElement, RoutedEventHandler>();

        #region Command Property

        public static readonly DependencyProperty CommandProperty = 
            DependencyProperty.RegisterAttached(
                "Command",
                typeof(ICommand),
                typeof(CommandBehavior),
                new FrameworkPropertyMetadata() {
                    DefaultValue = null,
                    PropertyChangedCallback = new PropertyChangedCallback(OnCommandPropertyChanged)
                }
            );
        public static ICommand GetCommand(UIElement obj) {
            return (ICommand)obj.GetValue(CommandProperty);
        }
        public static void SetCommand(UIElement obj, ICommand value) {
            obj.SetValue(CommandProperty, value);
        }

        #endregion

        #region Event Property
        public static readonly DependencyProperty EventProperty = 
            DependencyProperty.RegisterAttached(
                "Event",
                typeof(RoutedEvent),
                typeof(CommandBehavior),
                new FrameworkPropertyMetadata() {
                    DefaultValue = null,
                    PropertyChangedCallback = new PropertyChangedCallback(OnEventPropertyChanged)
                }
            );
        public static RoutedEvent GetEvent(DependencyObject obj) {
            return (RoutedEvent)obj.GetValue(EventProperty);
        }
        public static void SetEvent(DependencyObject obj, RoutedEvent value) {
            obj.SetValue(EventProperty, value);
        }

        #endregion

        #region CommandParameter Property

        public static readonly DependencyProperty CommandParameterProperty = 
            DependencyProperty.RegisterAttached(
                "CommandParameter",
                typeof(object),
                typeof(CommandBehavior),
                new FrameworkPropertyMetadata(null)
            );
        public static object GetCommandParameter(UIElement obj) {
            return obj.GetValue(CommandParameterProperty);
        }
        public static void SetCommandParameter(UIElement obj, object value) {
            obj.SetValue(CommandParameterProperty, value);
        }

        #endregion

        #region CallBacks

        private static void OnCommandPropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args) {
            UIElement element = obj as UIElement;
            ICommand oldCommand = args.OldValue as ICommand;
            ICommand newCommand = args.NewValue as ICommand;
            RoutedEvent routedEvent = element.GetValue(EventProperty) as RoutedEvent;
            object commandParameter = element.GetValue(CommandParameterProperty);

            Detach(element, routedEvent, oldCommand);
            Attach(element, routedEvent, newCommand, commandParameter);
        }

        private static void OnEventPropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args) {
            UIElement element = obj as UIElement;
            RoutedEvent oldEvent = args.OldValue as RoutedEvent;
            RoutedEvent newEvent = args.NewValue as RoutedEvent;
            ICommand command = element.GetValue(CommandProperty) as ICommand;
            object commandParameter = element.GetValue(CommandParameterProperty);

            Detach(element, oldEvent, command);
            Attach(element, newEvent, command, commandParameter);
        }

        #endregion

        private static void Attach(UIElement element, RoutedEvent Event, ICommand command, object commandParameter) {
            if (Event != null && element != null && command != null) {
                RoutedEventHandler InvokeCommandHandler = new RoutedEventHandler(delegate {
                    command.Execute(commandParameter);
                });
                handlers.Add(element, InvokeCommandHandler);
                element.AddHandler(Event, InvokeCommandHandler);
            }
        }

        private static void Detach(UIElement element, RoutedEvent Event, ICommand command) {
            if (Event != null && element != null && command != null) {
                RoutedEventHandler handler = handlers[element];
                if (handler != null) {
                    element.RemoveHandler(Event, handler);
                    handlers.Remove(element);
                }
            }
        }
    }
}
//public static class MyProperty {
//    public static readonly DependencyProperty ParameterABC = 
//        DependencyProperty.RegisterAttached(
//            "Parameter",
//            typeof(Object),
//            typeof(MyProperty),
//            new FrameworkPropertyMetadata(null)
//        );
//    public static Object GetParameter(UIElement obj) {
//        return obj.GetValue(ParameterABC);
//    }
//    public static void SetParameter(UIElement obj, Object value) {
//        obj.SetValue(ParameterABC, value);
//    }
//}
//}
